Puppet::Type.type(:ipset).provide(:ipset) do
  commands :ipset => '/sbin/ipset'

  def create
    ipset('create',@resource[:name],@resource[:type])
    [@resource[:set]].flatten.each do |item|
      ipset('add',@resource[:name],item)
    end
    flush
  end

  def destroy
    ipset('destroy',@resource[:name])
  end

  def exists?
    begin
      ipset('-n','list',@resource[:name])
    rescue
      false
    end
  end

  def set
    ipsetlist.split(/\n/).find_all do |line|
      line =~ /^add #{@resource[:name]}/
    end.collect do |line| 
      if line =~ /^add #{@resource[:name]} (.*)$/
        $1
      end
    end
  end

  def set=(val)
    ipset('flush',@resource[:name])
    [val].flatten.each do |item|
      ipset('add',@resource[:name],item)
    end
  end

  def type
    ipsetlist.split(/\n/).find{|line| line =~ /^create #{@resource[:name]}/} =~ /^create #{@resource[:name]} ([^\s]+)/
    $1
  end

  def type=(val)
    destroy
    create
  end

  def ipsetlist
    ipset('-s','-o','save','list',@resource[:name])
  end

  def flush
    ipset('-o','save','-f','/etc/sysconfig/ipset','list')
  end

  def self.instances
    ipset('-s','-n','list').split(/\n/).collect do |name|
      new({:name => name, :title => name})
    end
  end
end
